(function () {
  /**
   * Plugin factory function. This function is called when 
   * akamai.amp.AMP.create is called and the config object 
   * contains the plugin's key.
   *
   * @param {Player} player
   *    The AMP instance.
   *
   * @param {Object} config
   *    The plugin configuration object
   *
   * @param {String} key
   *    The key used to register the plugin
   *
   * @returns {Promise<Object>}
  */
  function createMuxPlugin(player, config) {
    return new Promise(function (resolve, reject) {
      var element = player.container;
      player.addEventListener("ready", function (event) {
        var ads = player.ads || {getEnabled: function () { return false; }, getInProgress: function () { return false; }};
        
        function mapEvents(target, map, isAdEvent) {
          for (var muxEvent in map) {
            target.addEventListener(map[muxEvent], function (element, muxEvent, playerEvent) {
              if (!isAdEvent && ads.getInProgress())
                return;
                
              if (!/timeupdate/.test(muxEvent)) 
                console.log("mux.emit", muxEvent); // TODO: Replace with player.logger.log
                
              mux.emit(element, muxEvent);
            }.bind(null, element, muxEvent));
          }
        }
        
        mapEvents(player, {
          "play": "play",
          "playing": "playing",
          "pause": "pause",
          "timeupdate": "timeupdate",
          "seeking": "seeking",
          "seeked": "seeked",
          "error": "error",
          "ended": "ended",
          "stalled": "stalled"
        });
        
        player.addEventListener("mediachange", function (event) {
          var media = event.detail;
          console.log("mux.emit", "videochange");
          mux.emit(element, "videochange", {
            video_id: media.guid,
            video_title: media.title,
            video_duration: media.duration,
            video_is_live: (media.temporalType == "live"),
            video_poster_url: media.poster,
            video_source_url: media.src,
            video_source_mime_type: media.type
          });
        });
        
        if (ads.getEnabled()) {
          mapEvents(ads, {
            "adrequest": "request",
            "adresponse": "response",
            "adbreakstart": "breakstart",
            "adplay": "play",
            "adplaying": "playing",
            "adpause": "pause",
            "adfirstquartile": "firstquartile",
            "admidpoint": "midpoint",
            "adthirdquartile": "thirdquartile",
            "adended": "ended",
            "adbreakend": "breakend",
            "aderror": "error"
          }, true);
          
          mapEvents(ads, {
            "adplay": "breakstart"
          }, true);
        }
        
        mux.emit(element, "playerready");
      });
      
      var plugin = {
        config: config,
        sdk: mux,
        data: Object.assign({
          player_software_name: "AMP",
          player_software_version: player.version,
          player_mux_plugin_name: "amp-mux",
          player_mux_plugin_version: "0.1.0",
          player_init_time: player.initTime,
        }, config.data),
        debug: player.debug,
        getStateData: function () {
          var error = player.error || {};
          var mediaElement = player.mediaElement;
          return {
            video_language_code: player.language,
            video_source_duration: isNaN(mediaElement.duration) ? undefined : Math.round(mediaElement.duration * 1000),
            video_source_height: mediaElement.videoHeight,
            video_source_width: mediaElement.videoWidth,
            player_is_paused: player.paused,
            player_playhead_time: this.getPlayheadTime(),
            player_is_fullscreen: (player.displayState == "fullscreen"),
            player_autoplay_on: player.autoplay,
            player_preload_on: mediaElement.preload,
            player_width: element.clientWidth,
            player_height: element.clientHeight,
            player_language_code: player.language,
            player_error_code: error.code,
            player_error_message: error.message
          };
        },
        getPlayheadTime: function () {
          return Math.round(player.getCurrentTime() * 1000);
        }
      };
      
      console.log("MUX INIT"); // TODO: Replace with player.logger.log
      mux.init(element, plugin);
      resolve(plugin);
    });
  }

  /**
   * Registers a plugin factory function. This function is 
   * called when akamai.amp.AMP.create is called and the 
   * config object contains the plugin's key.
   *
   * @param {String} key
   *    The plugin's key. Used to configure the plugin.
   *
   * @param {Function} factory
   *    The factory function used to create plugin.
   *
   * @param {String|Array.<String>} [mode=["html", "flash"]]
   *    Optional player mode(s) in which the plugin can be used.
  */
  akamai.amp.AMP.registerPlugin("mux", createMuxPlugin);
}());